#!/usr/bin/env python
#
# S3Config
# Copyright (c) 2014, Giacomo Marinangeli <jibbolo@gmail.com>
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library.

import re, os
from .parser import Parser
from .aws import fetch_file

class S3Config(object):
    
    re_s3_url = re.compile('^s3://([^/]+)/(.+)$')
    
    def __init__(self, url_s3, schema=None, aws_access=None, aws_secret=None):
        self.url_s3 = url_s3
        self.aws_access = aws_access
        self.aws_secret = aws_secret
        self.parser = Parser()
        self.bucket, self.key = self.parse_url()
        if schema is None:
            self.schema = self.schema_from_key(self.key)
        else:
            self.schema = schema
    
    @staticmethod
    def schema_from_key(key):
        try:
            return os.path.splitext(key)[1].replace(".","")
        except IndexError:
            return ""
            
    def fetch(self):
        return fetch_file(self.bucket,self.key,self.aws_access,self.aws_secret)
        
    def parse_data(self,data):
        parser = Parser(self.schema)
        try:
            return parser.parse(data)
        except parser.ParserError:
            return None
        
    def parse_url(self):
        try:
            return self.re_s3_url.findall(self.url_s3)[0]
        except IndexError:
            return None


    def read(self):
        data = self.fetch()
        return self.parse_data(data)


#!/usr/bin/env python
#
# S3Config
# Copyright (c) 2014, Giacomo Marinangeli <jibbolo@gmail.com>
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library.

import json
import yaml

class Parser(object):
    
    class ParserError(Exception): 
        pass
    
    def __init__(self, schema="json"):
        self.schema = schema
    
    def parse(self,data):
        if self.schema == "json":
            return self.parse_json(data)
        elif self.schema == "yalm":
            return self.parse_yalm(data)
        else:
            raise ValueError("Invalid Schma: {}".format(self.schema))
    
    def parse_yalm(self,data):
        try:
            return yaml.load(data)
        except Exception, e:
            raise self.ParserError("Invalid yalm format", e)
    
    def parse_json(self,data):
        try:
            return json.loads(data)
        except Exception, e:
            raise self.ParserError("Invalid json format", e)



#!/usr/bin/env python
#
# S3Config
# Copyright (c) 2014, Giacomo Marinangeli <jibbolo@gmail.com>
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library.

from boto.s3.connection import S3Connection
from boto.s3.key import Key
from boto.s3.bucket import Bucket

def fetch_file(bucket_name,key_name,aws_access=None,aws_secret=None):
    connection = S3Connection(aws_access,aws_secret)
    bucket = connection.get_bucket(bucket_name)
    key = bucket.get_key(key_name)
    return key.get_contents_as_string()



from distutils.core import setup
setup(
  name = 's3config',
  packages = ['s3config'], # this must be the same as the name above
  version = '0.1',
  description = 'Read config file from Amazon S3 url',
  author = 'Giacomo Marinangeli',
  author_email = 'jibbolo@gmail.com',
  url = 'https://bitbucket.org/jibbolo/s3config', # use the URL to the github repo
  download_url = 'https://bitbucket.org/jibbolo/s3config/get/v0.1.tar.gz', # I'll explain this in a second
  keywords = ['config', 's3', 'aws', 'configuration','json','yalm'], # arbitrary keywords
  classifiers = [],
  install_requires=[
      'PyYAML >= 3.11',
      'boto >= 2.28.0',
      'wsgiref >= 0.1.2',
  ],
)


import unittest
from s3config import S3Config, Parser

class TestParser(unittest.TestCase):
    
    def setUp(self):
        self.end_data = {
            'name': 'Vorlin Laruknuzum', 
            'gold': 423, 
            'float': 42.3, 
            'title': 'Acolyte', 
            'sp': [1, 13], 
            'inventory': [
                'a Holy Book of Prayers (Words of Wisdom)', 
                'an Azure Potion of Cure Light Wounds', 
                'a Silver Wand of Wonder'],
            }
        
    def test_invalid_schema(self):
        parser = Parser(schema="an_invalid_schema")
        with self.assertRaises(ValueError):
            parser.parse('123')
            
    def test_json(self):
        data = """
        {
            "name": "Vorlin Laruknuzum", 
            "gold": 423, 
            "float": 42.3,
            "title": "Acolyte", 
            "sp": [1, 13], 
            "inventory": [
                "a Holy Book of Prayers (Words of Wisdom)", 
                "an Azure Potion of Cure Light Wounds", 
                "a Silver Wand of Wonder"
            ]
        }
        """
        parser = Parser(schema="json")
        self.assertTrue(parser.parse(data) == self.end_data)
        
    def test_yalm(self):
        data = """
        name: Vorlin Laruknuzum
        title: Acolyte
        sp: [1, 13]
        gold: 423
        float: 42.3
        inventory:
        - a Holy Book of Prayers (Words of Wisdom)
        - an Azure Potion of Cure Light Wounds
        - a Silver Wand of Wonder
        """
        parser = Parser(schema="yalm")
        self.assertTrue(parser.parse(data) == self.end_data)
        

class TestS3Config(unittest.TestCase):
    def test_urls(self):
        self.assertEqual(S3Config("s3://asd_backer/mi/ofile.json").parse_url(), ("asd_backer","mi/ofile.json"))
        self.assertEqual(S3Config("s3://asdbacker/mi_ofile.json").parse_url(), ("asdbacker","mi_ofile.json"))
        self.assertEqual(S3Config("s3://asd-backer/mi-ofile").parse_url(),("asd-backer","mi-ofile"))
    def test_schema(self):
        self.assertEqual(S3Config.schema_from_key("key.json"), "json")
        self.assertEqual(S3Config.schema_from_key("key.yalm"), "yalm")
        self.assertEqual(S3Config.schema_from_key("key..test"), "test")
        self.assertEqual(S3Config.schema_from_key("key"), "")
    def test_s3config(self):
        config = S3Config("s3://s3config/config.json").read()
        self.assertIsInstance(config,dict)
        self.assertTrue(len(config.keys())==6)
        config = S3Config("s3://s3config/config.yalm").read()
        self.assertIsInstance(config,dict)
        self.assertTrue(len(config.keys())==6)

if __name__ == '__main__':
    unittest.main()
    